==> org-lsst-ccs-subsystem-image-handling-main-1.1.0-dev-20230608.201446-37-sources.jar: org/lsst/ccs/subsystem/imagehandling/ImageHandlingConfig.java <==
package org.lsst.ccs.subsystem.imagehandling;

import java.io.File;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import org.lsst.ccs.commons.annotations.ConfigurationParameter;
import org.lsst.ccs.config.ConfigurationBulkChangeHandler;
import org.lsst.ccs.daq.ims.channel.FitsIntWriter;
import org.lsst.ccs.subsystem.imagehandling.config.WaitForHeaderService;
import org.lsst.ccs.subsystem.imagehandling.data.AdditionalFile;
import org.lsst.ccs.utilities.location.LocationSet;
import org.lsst.ccs.utilities.location.SensorLocationSet;
import org.lsst.ccs.utilities.pattern.FileNamePatternUtils;

/**
 *
 * @author tonyj
 */
@SuppressWarnings("FieldMayBeFinal")
public class ImageHandlingConfig implements ConfigurationBulkChangeHandler {

    private static final String TMP_POSTFIX = ".$tmp$";

    @ConfigurationParameter(category = "FitsHandling", units = "unitless")
    private volatile String FITSRootDirectory = System.getProperty("java.io.tmpdir");

    @ConfigurationParameter(category = "FitsHandling", units = "unitless")
    private volatile String FITSDirectoryPattern = "${ImageDate}/${ImageName}";

    @ConfigurationParameter(category = "FitsHandling", units = "unitless")
    private volatile String FITSFilePattern = "${ImageName}_${RaftBay}_${CCDSlot}.fits";

    @ConfigurationParameter(category = "FitsHandling", units = "unitless")
    private volatile boolean FITSAutoSave = true;

    @ConfigurationParameter(category = "FitsHandling", units = "unitless")
    private volatile boolean useTempFile = true;

    @ConfigurationParameter(category = "FitsHandling", units = "unitless")
    private volatile String tempFileRelativeLocation = ".";

    @ConfigurationParameter(category = "FitsHandling", units = "unitless", maxLength = 10)
    @SuppressWarnings("VolatileArrayField")
    private volatile String[] subsystemsToClearOnNewRun = {"bot-bench", "ccob", "bot-motorpltform"};

    @ConfigurationParameter(category = "FitsHandling", units = "unitless")
    private volatile WaitForHeaderService waitForHeaderService = WaitForHeaderService.AUTO;

    @ConfigurationParameter(category = "FitsHandling", units = "unitless")
    volatile boolean raiseAlertOnMissingOrBadHeaderServiceData = false;   

    @ConfigurationParameter(category = "DAQ", units = "unitless")
    private volatile String daqPartition = "dev";

    @ConfigurationParameter(category = "DAQ", units = "unitless")
    private volatile String daqFolder = "raw";

    @ConfigurationParameter(category = "DAQ", units = "unitless")
    private volatile String locations = "R22/Reb0,R22/Reb1,R22/Reb2";

    @ConfigurationParameter(category = "DAQ", units = "unitless")
    private volatile int daqThreads = 4;

    @ConfigurationParameter(category = "DAQ", units = "unitless")
    private volatile boolean useStreaming = false;

    @ConfigurationParameter(category = "DAQ", units = "unitless")
    private volatile boolean checkForBadPixels = false;

    @ConfigurationParameter(category = "DAQ", units = "unitless")
    private volatile int badPixelLowThreshold = 10000;

    @ConfigurationParameter(category = "DAQ", units = "unitless")
    private volatile int badPixelHighThreshold = 240000;

    @ConfigurationParameter(category = "DAQ", units = "unitless")
    private volatile int badPixelWarningLevel = 0;

    @ConfigurationParameter(category = "DAQ", units = "unitless")
    private volatile int badPixelAlarmLevel = 20000;

    @ConfigurationParameter(category = "Guider", units = "unitless")
    private volatile String guiderPartition = "";

    @ConfigurationParameter(category = "Guider", units = "unitless")
    private volatile String guiderLocations = "R00SG0,R00SG1";

    @ConfigurationParameter(category = "Commands", units = "unitless", maxLength = 20)
    private volatile List<String> commands = Collections.<String>emptyList();

    @ConfigurationParameter(category = "Commands", units = "unitless", maxLength = 20)
    private volatile Map<String, String> environment = Collections.<String, String>emptyMap();

    @ConfigurationParameter(category = "Commands", units = "unitless")
    private volatile String currentDirectory = ".";

    @ConfigurationParameter(category = "Commands", units = "unitless")
    private volatile String logDirectory = System.getProperty("java.io.tmpdir");

    @ConfigurationParameter(category = "Commands", units = "unitless", maxLength = 20)
    private volatile List<String> additionalFileCommands = Collections.<String>singletonList("env");

    @Override
    public void validateBulkChange(Map<String, Object> parametersView) {
        String dir = parametersView.get("FITSRootDirectory").toString();
        File dirFile = new File(dir);
        checkWritable("FITSRootDirectory", dirFile);

        String logDir = parametersView.get("logDirectory").toString();
        File logDirFile = new File(logDir);
        checkWritable("logDirectory", logDirFile);

        // Validate that the locations syntax is valid.
        LocationSet.of(parametersView.get("locations").toString());
        // Validate that the guider locations are valid
        SensorLocationSet.of(parametersView.get("guiderLocations").toString());
    }

    private void checkWritable(String dirName, File dirFile) throws IllegalArgumentException {
        dirFile.mkdirs();
        if (!dirFile.isDirectory()) {
            throw new IllegalArgumentException(dirName + ": Not a directory");
        }
        if (!dirFile.canWrite()) {
            throw new IllegalArgumentException(dirName + ": Not writable");
        }
    }

    public String getFITSRootDirectory() {
        return FITSRootDirectory;
    }

    public String getFITSDirectoryPattern() {
        return FITSDirectoryPattern;
    }

    public String getFITSFilePattern() {
        return FITSFilePattern;
    }

    public boolean isFITSAutoSave() {
        return FITSAutoSave;
    }

    public String getDaqPartition() {
        return daqPartition;
    }

    LocationSet getLocationsToProcess() {
        return LocationSet.of(locations);
    }

    public String getDaqFolder() {
        return daqFolder;
    }

    public int getDaqThreads() {
        return daqThreads;
    }

    public boolean isUseTempFile() {
        return useTempFile;
    }

    public boolean isUseStreaming() {
        return useStreaming;
    }

    public List<String> getCommands() {
        return commands == null ? Collections.<String>emptyList() : commands;
    }

    public List<String> getAdditionalFileCommands() {
        return additionalFileCommands == null ? Collections.<String>emptyList() : additionalFileCommands;
    }
    
    boolean isPrimary() {
        return additionalFileCommands != null && !additionalFileCommands.isEmpty();
    }
    
    public Map<String, String> getEnvironment() {
        return environment == null ? Collections.<String, String>emptyMap() : environment;
    }

    public File getCurrentDirectory() {
        return new File(currentDirectory);
    }

    public File getLogDirectory() {
        return new File(logDirectory);
    }

    FitsIntWriter.FileNamer getFileNamer(Map<File, File> tmpMap) {

        return (Map<String, Object> props) -> {
            File file = getFitsFile(props);
            // Note, this is the name the file will have (after it is closed and renamed)
            // rather than the temporary name it has initially. 
            props.put("OriginalFileName", file.getName());
            if (isUseTempFile()) {
                File tmpFile = getTempFitsFile(props);
                tmpMap.put(tmpFile, file);
                return tmpFile;
            } else {
                return file;
            }
        };
    }

    File getFitsFile(Map<String, Object> data) {
        // Convert input data to a map of String, String TODO: Move this to FileNamePatternUtils
        Map<String, String> props = data.entrySet().stream().collect(Collectors.toMap(e -> e.getKey(), e -> e.getValue().toString()));
        File root = new File(FITSRootDirectory);
        String fileDir = FileNamePatternUtils.resolveFileName(FITSDirectoryPattern, props);
        File dir = new File(root, fileDir);
        String fileName = FileNamePatternUtils.resolveFileName(FITSFilePattern, props);
        File result = new File(dir, fileName);
        result.getParentFile().mkdirs();
        return result;
    }

    File getTempFitsFile(Map<String, Object> data) {
        // Convert input data to a map of String, String TODO: Move this to FileNamePatternUtils
        Map<String, String> props = data.entrySet().stream().collect(Collectors.toMap(e -> e.getKey(), e -> e.getValue().toString()));
        File root = new File(FITSRootDirectory);
        String fileDir = FileNamePatternUtils.resolveFileName(FITSDirectoryPattern, props);
        File dir = new File(root, fileDir);
        dir = new File(dir, tempFileRelativeLocation);
        String fileName = FileNamePatternUtils.resolveFileName(FITSFilePattern, props);
        fileName += TMP_POSTFIX;
        File result = new File(dir, fileName);
        result.getParentFile().mkdirs();
        return result;
    }

    File getAdditionalFileName(AdditionalFile additionalFile, Map<String, String> props) {
        File root = new File(FITSRootDirectory);
        String fileDir = FileNamePatternUtils.resolveFileName(FITSDirectoryPattern, props);
        File dir = new File(root, fileDir);
        File result = new File(dir, additionalFile.getFileName());
        result.getParentFile().mkdirs();
        return result;
    }

    String[] getSubsystemsToClear() {
        return subsystemsToClearOnNewRun;
    }

    public int getBadPixelLowThreshold() {
        return badPixelLowThreshold;
    }

    public int getBadPixelHighThreshold() {
        return badPixelHighThreshold;
    }

    public int getBadPixelWarningLevel() {
        return badPixelWarningLevel;
    }

    public int getBadPixelAlarmLevel() {
        return badPixelAlarmLevel;
    }

    public boolean isCheckForBadPixels() {
        return checkForBadPixels;
    }

    public WaitForHeaderService getWaitForHeaderService() {
        return waitForHeaderService;
    }

    public String getGuiderPartition() {
        return guiderPartition;
    }

    public SensorLocationSet getGuiderLocations() {
        return SensorLocationSet.of(guiderLocations);
    }
    
    public boolean getRaiseAlertOnMissingOrBadHeaderServiceData() {
        return raiseAlertOnMissingOrBadHeaderServiceData;
    }
}

==> org-lsst-ccs-subsystem-image-handling-main-1.1.2-sources.jar: org/lsst/ccs/subsystem/imagehandling/ImageHandlingConfig.java <==
package org.lsst.ccs.subsystem.imagehandling;

import java.io.File;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import org.lsst.ccs.commons.annotations.ConfigurationParameter;
import org.lsst.ccs.config.ConfigurationBulkChangeHandler;
import org.lsst.ccs.daq.ims.channel.FitsIntWriter;
import org.lsst.ccs.imagenaming.ImageName;
import org.lsst.ccs.subsystem.imagehandling.config.WaitForHeaderService;
import org.lsst.ccs.subsystem.imagehandling.data.AdditionalFile;
import org.lsst.ccs.utilities.location.LocationSet;
import org.lsst.ccs.utilities.location.SensorLocationSet;
import org.lsst.ccs.utilities.pattern.FileNamePatternUtils;

/**
 *
 * @author tonyj
 */
@SuppressWarnings("FieldMayBeFinal")
public class ImageHandlingConfig implements ConfigurationBulkChangeHandler {

    private static final String TMP_POSTFIX = ".$tmp$";

    @ConfigurationParameter(category = "FitsHandling", units = "unitless")
    private volatile String FITSRootDirectory = System.getProperty("java.io.tmpdir");

    @ConfigurationParameter(category = "FitsHandling", units = "unitless")
    private volatile String FITSDirectoryPattern = "${ImageDate}/${ImageName}";

    @ConfigurationParameter(category = "FitsHandling", units = "unitless")
    private volatile String FITSFilePattern = "${ImageName}_${RaftBay}_${CCDSlot}.fits";

    @ConfigurationParameter(category = "FitsHandling", units = "unitless")
    private volatile boolean FITSAutoSave = true;

    @ConfigurationParameter(category = "FitsHandling", units = "unitless")
    private volatile boolean useTempFile = true;

    @ConfigurationParameter(category = "FitsHandling", units = "unitless")
    private volatile String tempFileRelativeLocation = ".";

    @ConfigurationParameter(category = "FitsHandling", units = "unitless", maxLength = 10)
    @SuppressWarnings("VolatileArrayField")
    private volatile String[] subsystemsToClearOnNewRun = {"bot-bench", "ccob", "bot-motorpltform"};

    @ConfigurationParameter(category = "FitsHandling", units = "unitless")
    private volatile WaitForHeaderService waitForHeaderService = WaitForHeaderService.AUTO;

    @ConfigurationParameter(category = "FitsHandling", units = "unitless")
    volatile boolean raiseAlertOnMissingOrBadHeaderServiceData = false;   

    @ConfigurationParameter(category = "DAQ", units = "unitless")
    private volatile String daqPartition = "dev";

    @ConfigurationParameter(category = "DAQ", units = "unitless")
    private volatile String daqFolder = "raw";

    @ConfigurationParameter(category = "DAQ", units = "unitless")
    private volatile String locations = "R22/Reb0,R22/Reb1,R22/Reb2";

    @ConfigurationParameter(category = "DAQ", units = "unitless")
    private volatile int daqThreads = 4;

    @ConfigurationParameter(category = "DAQ", units = "unitless")
    private volatile boolean useStreaming = false;

    @ConfigurationParameter(category = "DAQ", units = "unitless")
    private volatile boolean checkForBadPixels = false;

    @ConfigurationParameter(category = "DAQ", units = "unitless")
    private volatile int badPixelLowThreshold = 10000;

    @ConfigurationParameter(category = "DAQ", units = "unitless")
    private volatile int badPixelHighThreshold = 240000;

    @ConfigurationParameter(category = "DAQ", units = "unitless")
    private volatile int badPixelWarningLevel = 0;

    @ConfigurationParameter(category = "DAQ", units = "unitless")
    private volatile int badPixelAlarmLevel = 20000;

    @ConfigurationParameter(category = "Guider", units = "unitless")
    private volatile String guiderPartition = "";

    @ConfigurationParameter(category = "Guider", units = "unitless")
    private volatile String guiderLocations = "R00SG0,R00SG1";

    @ConfigurationParameter(category = "Commands", units = "unitless", maxLength = 20)
    private volatile List<String> commands = Collections.<String>emptyList();

    @ConfigurationParameter(category = "Commands", units = "unitless", maxLength = 20)
    private volatile Map<String, String> environment = Collections.<String, String>emptyMap();

    @ConfigurationParameter(category = "Commands", units = "unitless")
    private volatile String currentDirectory = ".";

    @ConfigurationParameter(category = "Commands", units = "unitless")
    private volatile String logDirectory = System.getProperty("java.io.tmpdir");

    @ConfigurationParameter(category = "Commands", units = "unitless", maxLength = 20)
    private volatile List<String> additionalFileCommands = Collections.<String>singletonList("env");

    @ConfigurationParameter(category = "Commands", units = "unitless", maxLength = 20)
    private volatile String additionalFileFilter = "";
    
    @Override
    public void validateBulkChange(Map<String, Object> parametersView) {
        String dir = parametersView.get("FITSRootDirectory").toString();
        File dirFile = new File(dir);
        checkWritable("FITSRootDirectory", dirFile);

        String logDir = parametersView.get("logDirectory").toString();
        File logDirFile = new File(logDir);
        checkWritable("logDirectory", logDirFile);

        // Validate that the locations syntax is valid.
        LocationSet.of(parametersView.get("locations").toString());
        // Validate that the guider locations are valid
        SensorLocationSet.of(parametersView.get("guiderLocations").toString());
    }

    private void checkWritable(String dirName, File dirFile) throws IllegalArgumentException {
        dirFile.mkdirs();
        if (!dirFile.isDirectory()) {
            throw new IllegalArgumentException(dirName + ": Not a directory");
        }
        if (!dirFile.canWrite()) {
            throw new IllegalArgumentException(dirName + ": Not writable");
        }
    }

    public String getFITSRootDirectory() {
        return FITSRootDirectory;
    }

    public String getFITSDirectoryPattern() {
        return FITSDirectoryPattern;
    }

    public String getFITSFilePattern() {
        return FITSFilePattern;
    }

    public boolean isFITSAutoSave() {
        return FITSAutoSave;
    }

    public String getDaqPartition() {
        return daqPartition;
    }

    LocationSet getLocationsToProcess() {
        return LocationSet.of(locations);
    }

    public String getDaqFolder() {
        return daqFolder;
    }

    public int getDaqThreads() {
        return daqThreads;
    }

    public boolean isUseTempFile() {
        return useTempFile;
    }

    public boolean isUseStreaming() {
        return useStreaming;
    }

    public List<String> getCommands() {
        return commands == null ? Collections.<String>emptyList() : commands;
    }

    public List<String> getAdditionalFileCommands() {
        return additionalFileCommands == null ? Collections.<String>emptyList() : additionalFileCommands;
    }
    
    boolean canHandleAdditionalFiles() {
        return additionalFileCommands != null && !additionalFileCommands.isEmpty();
    }
    
    boolean acceptAdditionalFileObsID(ImageName obsId) {
        return additionalFileFilter == null || additionalFileFilter.isBlank() || obsId.toString().startsWith(additionalFileFilter);
    }
    
    public Map<String, String> getEnvironment() {
        return environment == null ? Collections.<String, String>emptyMap() : environment;
    }

    public File getCurrentDirectory() {
        return new File(currentDirectory);
    }

    public File getLogDirectory() {
        return new File(logDirectory);
    }

    FitsIntWriter.FileNamer getFileNamer(Map<File, File> tmpMap) {

        return (Map<String, Object> props) -> {
            File file = getFitsFile(props);
            // Note, this is the name the file will have (after it is closed and renamed)
            // rather than the temporary name it has initially. 
            props.put("OriginalFileName", file.getName());
            if (isUseTempFile()) {
                File tmpFile = getTempFitsFile(props);
                tmpMap.put(tmpFile, file);
                return tmpFile;
            } else {
                return file;
            }
        };
    }

    File getFitsFile(Map<String, Object> data) {
        // Convert input data to a map of String, String TODO: Move this to FileNamePatternUtils
        Map<String, String> props = data.entrySet().stream().collect(Collectors.toMap(e -> e.getKey(), e -> e.getValue().toString()));
        File root = new File(FITSRootDirectory);
        String fileDir = FileNamePatternUtils.resolveFileName(FITSDirectoryPattern, props);
        File dir = new File(root, fileDir);
        String fileName = FileNamePatternUtils.resolveFileName(FITSFilePattern, props);
        File result = new File(dir, fileName);
        result.getParentFile().mkdirs();
        return result;
    }

    File getTempFitsFile(Map<String, Object> data) {
        // Convert input data to a map of String, String TODO: Move this to FileNamePatternUtils
        Map<String, String> props = data.entrySet().stream().collect(Collectors.toMap(e -> e.getKey(), e -> e.getValue().toString()));
        File root = new File(FITSRootDirectory);
        String fileDir = FileNamePatternUtils.resolveFileName(FITSDirectoryPattern, props);
        File dir = new File(root, fileDir);
        dir = new File(dir, tempFileRelativeLocation);
        String fileName = FileNamePatternUtils.resolveFileName(FITSFilePattern, props);
        fileName += TMP_POSTFIX;
        File result = new File(dir, fileName);
        result.getParentFile().mkdirs();
        return result;
    }

    File getAdditionalFileName(AdditionalFile additionalFile, Map<String, String> props) {
        File root = new File(FITSRootDirectory);
        String fileDir = FileNamePatternUtils.resolveFileName(FITSDirectoryPattern, props);
        File dir = new File(root, fileDir);
        File result = new File(dir, additionalFile.getFileName());
        result.getParentFile().mkdirs();
        return result;
    }

    String[] getSubsystemsToClear() {
        return subsystemsToClearOnNewRun;
    }

    public int getBadPixelLowThreshold() {
        return badPixelLowThreshold;
    }

    public int getBadPixelHighThreshold() {
        return badPixelHighThreshold;
    }

    public int getBadPixelWarningLevel() {
        return badPixelWarningLevel;
    }

    public int getBadPixelAlarmLevel() {
        return badPixelAlarmLevel;
    }

    public boolean isCheckForBadPixels() {
        return checkForBadPixels;
    }

    public WaitForHeaderService getWaitForHeaderService() {
        return waitForHeaderService;
    }

    public String getGuiderPartition() {
        return guiderPartition;
    }

    public SensorLocationSet getGuiderLocations() {
        return SensorLocationSet.of(guiderLocations);
    }
    
    public boolean getRaiseAlertOnMissingOrBadHeaderServiceData() {
        return raiseAlertOnMissingOrBadHeaderServiceData;
    }
}
